Attribute VB_Name = "Thermo"

' Functions defined in this module are used when the M401 is performing extended calibration
' scan with subchan 0 as the offset, with thermocouple thermometer (M20B or M35B) connected
' AD 0 or AD 1 or both. Functions defined here include calculating cold junction temperature
' (sub channel 3), thermocouple temperature, linearizing the temperatures
' NOTE THE FUNCTIONS MUST BE CALLED AFTER SUBCHAN 0 HAS BEEN SUBTRACTED FROM ALL OTHER
' SUBCHANS
' All internal calculations in centigrade. Conversion to farenheight if selected is performed after
' linearization

Option Explicit

Private dblColdJuncTempAD0, dblColdJuncTempAD1 As Double

' Called by the main code to calculate all the tc temperatures
Public Function CalcAllTemps(Index As Integer)
Dim i, lCurScanOffset As Long
Dim Ad1SubChan0Offset, j As Long
Dim dblV, dblT As Double

'First calculate cold junction temperature

lCurScanOffset = 0
Ad1SubChan0Offset = IC_Array(Index).Ad1SubChan0Offset
For j = 0 To (IC_Array(Index).ulNumOfExScans - 1)
    If IC_Array(Index).ExtScan_AD0_Start = 0 And (IC_Array(Index).ExtScan_AD0_End > 2) Then
       'COLD JUNCTION temprature calculations
       dblColdJuncTempAD0 = IC_Array(Index).fExScans(lCurScanOffset + 3) / IC_Array(Index).fAD0_K(3) - 273
       IC_Array(Index).fExScans(lCurScanOffset + 3) = dblColdJuncTempAD0
      'AD chan 0 cal with subchan 0 as offset
       For i = 1 To IC_Array(Index).ExtScan_AD0_End
            If i <> 3 Then      'skip cold junction channel
                dblV = IC_Array(Index).fExScans(lCurScanOffset + i)
                dblV = dblV * IC_Array(Index).fAD0_G(i)
                dblT = Linearize(IC_Array(Index).fAD0_Type(i), dblColdJuncTempAD0, dblV)
                'update min, max
                If (IC_Array(Index).fAD0_SubChan_Min(i) > dblT) Then IC_Array(Index).fAD0_SubChan_Min(i) = dblT
                If (IC_Array(Index).fAD0_SubChan_Max(i) < dblT) Then IC_Array(Index).fAD0_SubChan_Max(i) = dblT
            End If
            If IC_Array(Index).bFaren <> 0 Then
                IC_Array(Index).fExScans(lCurScanOffset + i) = 1.8 * dblT + 32
            Else
                IC_Array(Index).fExScans(lCurScanOffset + i) = dblT
            End If
        Next i
     End If
                         
    If IC_Array(Index).ExtScan_AD1_Start = 0 And (IC_Array(Index).ExtScan_AD1_End > 2) Then
        'AD chan 1 cal with subchan 0 as offset
        'COLD JUNCTION temperature calculations
        dblColdJuncTempAD1 = (IC_Array(Index).fExScans(lCurScanOffset + Ad1SubChan0Offset + 3) / IC_Array(Index).fAD1_K(3)) - 273
        IC_Array(Index).fExScans(lCurScanOffset + Ad1SubChan0Offset + 3) = dblColdJuncTempAD1
        For i = 1 To IC_Array(Index).ExtScan_AD1_End
            If i <> 3 Then    ''skip cold junction channel
                dblV = IC_Array(Index).fExScans(lCurScanOffset + Ad1SubChan0Offset + i)
                dblV = dblV * IC_Array(Index).fAD1_G(i)
                dblT = Linearize(IC_Array(Index).fAD1_Type(i), dblColdJuncTempAD1, dblV)
                If IC_Array(Index).fAD1_SubChan_Min(i) > dblT Then IC_Array(Index).fAD1_SubChan_Min(i) = dblT
                If IC_Array(Index).fAD1_SubChan_Max(i) < dblT Then IC_Array(Index).fAD1_SubChan_Max(i) = dblT
            End If
            If IC_Array(Index).bFaren <> 0 Then
                IC_Array(Index).fExScans(lCurScanOffset + Ad1SubChan0Offset + i) = 1.8 * dblT + 32
            Else
                IC_Array(Index).fExScans(lCurScanOffset + Ad1SubChan0Offset + i) = dblT
            End If
        Next i
    End If
    lCurScanOffset = lCurScanOffset + IC_Array(Selected_Index).bNumOfExChans
Next j
End Function




Private Function Linearize(ByVal Thermo_Type As String, ByVal dblColdJuncTemp As Double, ByVal V As Double) As Double

If UCase(Thermo_Type) = "K" Then GoTo Thermo_K
If UCase(Thermo_Type) = "T" Then GoTo Thermo_T
If UCase(Thermo_Type) = "E" Then GoTo Thermo_E
If UCase(Thermo_Type) = "J" Then GoTo Thermo_J
Exit Function

'LINEARIZATION PROGRAM FOR TYPE K THERMOCOUPLES
Thermo_E:
  V = V + dblColdJuncTemp * 6.1
  If V < 366 Then Linearize = V / 6.1: Exit Function
  If V < 836 Then Linearize = 60 + (V - 366) / 6.72: Exit Function
  If V < 1487 Then Linearize = 130 + (V - 836) / 7.223: Exit Function
  If V < 2494 Then Linearize = 220 + (V - 1487) / 7.74: Exit Function
  If V < 5633 Then Linearize = 350 + (V - 2494) / 8.049: Exit Function
  If V < 6878 Then Linearize = 740 + (V - 5633) / 7.785: Exit Function
  Linearize = 900 + (V - 6878) / 7.6: Exit Function

'LINEARIZATION PROGRAM FOR TYPE J THERMOCOUPLES
Thermo_J:
  V = V + dblColdJuncTemp * 5.2
  If V < 411 Then Linearize = V / 5.2: Exit Function
  If V < 2738 Then Linearize = 79 + (V - 411) / 5.527: Exit Function
  If V < 3367 Then Linearize = 500 + (V - 2738) / 5.72: Exit Function
  If V < 3911 Then Linearize = 610 + (V - 3367) / 6.05: Exit Function
  Linearize = 700 + (V - 3911) / 6.33: Exit Function


'LINEARIZATION PROGRAM FOR TYPE E THERMOCOUPLES
Thermo_K:
  V = V + dblColdJuncTemp * 4.07
  If V < 326 Then Linearize = V / 4.07: Exit Function
  If V < 654 Then Linearize = 80 + (V - 326) / 4.11: Exit Function
  If V < 1137 Then Linearize = 160 + (V - 654) / 4.02: Exit Function
  If V < 1935 Then Linearize = 280 + (V - 1137) / 4.2: Exit Function
  If V < 2932 Then Linearize = 470 + (V - 1935) / 4.26: Exit Function
  If V < 3614 Then Linearize = 704 + (V - 2932) / 4.11: Exit Function
  If V < 4276 Then Linearize = 870 + (V - 3614) / 3.94: Exit Function
  If V < 4811 Then Linearize = 1038 + (V - 4276) / 3.77: Exit Function
  If V < 5241 Then Linearize = 1180 + (V - 4811) / 3.58: Exit Function
  Linearize = 1300 + (V - 5241) / 3.43: Exit Function

'LINEARIZATION PROGRAM FOR TYPE T THERMOCOUPLES
Thermo_S:
  V = V * 10 + dblColdJuncTemp * 5.54
  If V < 78 Then Linearize = V / 5.54: Exit Function
  If V < 154 Then Linearize = 14 + (V - 78) / 5.83: Exit Function
  If V < 266 Then Linearize = 27 + (V - 154) / 6.22: Exit Function
  If V < 405 Then Linearize = 45 + (V - 266) / 6.59: Exit Function
  If V < 502 Then Linearize = 66 + (V - 405) / 6.9: Exit Function
  If V < 623 Then Linearize = 80 + (V - 502) / 7.1: Exit Function
  If V < 810 Then Linearize = 97 + (V - 623) / 7.45: Exit Function
  If V < 989 Then Linearize = 122 + (V - 810) / 7.75: Exit Function
  If V < 1231 Then Linearize = 145 + (V - 989) / 8.08: Exit Function
  If V < 1499 Then Linearize = 175 + (V - 1231) / 8.350001: Exit Function
  If V < 1759 Then Linearize = 207 + (V - 1499) / 8.640001: Exit Function
  If V < 2015 Then Linearize = 237 + (V - 1759) / 8.8: Exit Function
  If V < 2277 Then Linearize = 266 + (V - 2015) / 9.020001: Exit Function
  If V < 2646 Then Linearize = 295 + (V - 2277) / 9.22: Exit Function
  If V < 3117 Then Linearize = 335 + (V - 2646) / 9.41: Exit Function
  If V < 3694 Then Linearize = 385 + (V - 3117) / 9.609999: Exit Function
  If V < 4283 Then Linearize = 445 + (V - 3694) / 9.810001: Exit Function
  If V < 4984 Then Linearize = 505 + (V - 4283) / 10.01: Exit Function
  If V < 5391 Then Linearize = 575 + (V - 4984) / 10.16: Exit Function
  If V < 5907 Then Linearize = 615 + (V - 5391) / 10.32: Exit Function
  If V < 6539 Then Linearize = 665 + (V - 5907) / 10.52: Exit Function
  If V < 7182 Then Linearize = 725 + (V - 6539) / 10.71: Exit Function
  If V < 7837 Then Linearize = 785 + (V - 7182) / 10.92: Exit Function
  Linearize = 845 + (V - 7837) / 11.1: Exit Function
'Linearize THERMOCOUPLES FOR TYPE S THERMOCOUPLES
Thermo_T:
  V = V + dblColdJuncTemp * 4.04
  If V < -112 Then Linearize = -30 + (V + 112) / 3.53: Exit Function
  If V < 0 Then Linearize = V / 3.74: Exit Function
  If V < 202 Then Linearize = V / 4.04: Exit Function
  If V < 474 Then Linearize = 50 + (V - 202) / 4.53: Exit Function
  If V < 822 Then Linearize = 110 + (V - 474) / 4.968: Exit Function
  If V < 1256 Then Linearize = 180 + (V - 822) / 5.425: Exit Function
  If V < 1780 Then Linearize = 260 + (V - 1256) / 5.825: Exit Function
  Linearize = 350 + (V - 1780) / 6.15: Exit Function
End Function
